@@ -105,18 +105,38 @@ struct icalparser_impl
 };
 
 
-char* strstrip (char *str)
+/*
+ * New version of strstrip() that does not move the pointer.
+ */
+void strstriplt(char *buf)
 {
-	if (!str)
-		return NULL;
-	while (*str == ' ')
-		str = str+1;
-	while (*str && str[strlen  (str)-1] == ' ')
-		str[strlen (str)-1] = 0;
-	return str;
+	size_t len;
+	int a;
+
+	if (buf==NULL) {
+		return;
+	}
+	if (buf[0] == 0) {
+		return;
+	}
+	len = strlen(buf);
+        while ((buf[0] != 0) && (isspace(buf[len - 1]))) {
+                buf[--len] = 0;
+	}
+	if (buf[0] == 0) {
+		return;
+	}
+	a = 0;
+        while ((buf[0]!=0) && (isspace(buf[a]))) {
+		a++;
+	}
+	if (a > 0) {
+                memmove(buf, &buf[a], len - a + 1);
+	}
 }
 
 
+
 icalparser* icalparser_new(void)
 {
     struct icalparser_impl* impl = 0;
@@ -299,6 +319,21 @@ char* parser_get_next_paramvalue(char* line, char **end)
 }
 #endif
 
+char* icalparser_get_value(char* line, char **end, icalvalue_kind kind)
+{
+    char *str;
+    size_t length = strlen(line);
+
+    if (length == 0){
+        return 0;
+    } 
+
+    *end = line+length;
+    str = make_segment(line, *end);
+
+    return str;
+}
+
 /**
    A property may have multiple values, if the values are seperated by
    commas in the content line. This routine will look for the next
@@ -640,7 +675,6 @@ icalcomponent* icalparser_add_line(icalparser* parser,
                                        char* line)
 { 
     char *str;
-	char *strptr;
     char *end;
     int vcount = 0;
     icalproperty *prop;
@@ -681,7 +715,8 @@ icalcomponent* icalparser_add_line(icalparser* parser,
 	}
 	tail = 0;
 	parser->state = ICALPARSER_ERROR;
-	icalmemory_free_buffer(str), str = NULL;
+	icalmemory_free_buffer(str);
+	str = NULL;
 	return 0; 
     }
 
@@ -696,7 +731,8 @@ icalcomponent* icalparser_add_line(icalparser* parser,
 	icalcomponent *c;
         icalcomponent_kind comp_kind;
 
-	icalmemory_free_buffer(str), str = NULL;
+	icalmemory_free_buffer(str);
+	str = NULL;
 
 	parser->level++;
 	str = parser_get_next_value(end,&end, value_kind);
@@ -725,13 +761,15 @@ icalcomponent* icalparser_add_line(icalparser* parser,
 
 	parser->state = ICALPARSER_BEGIN_COMP;
 
-	icalmemory_free_buffer(str), str = NULL;
+	icalmemory_free_buffer(str);
+	str = NULL;
 	return 0;
 
     } else if (strcasecmp(str,"END") == 0 ) {
 	icalcomponent* tail;
 
-	icalmemory_free_buffer(str), str = NULL;
+	icalmemory_free_buffer(str);
+	str = NULL;
 	parser->level--;
 
 	str = parser_get_next_value(end,&end, value_kind);
@@ -746,7 +784,8 @@ icalcomponent* icalparser_add_line(icalparser* parser,
 	} 
 
 	tail = 0;
-	icalmemory_free_buffer(str), str = NULL;
+	icalmemory_free_buffer(str);
+	str = NULL;
 
 	/* Return the component if we are back to the 0th level */
 	if (parser->level == 0){
@@ -778,7 +817,8 @@ icalcomponent* icalparser_add_line(icalparser* parser,
 
     if(pvl_data(pvl_tail(parser->components)) == 0){
 	parser->state = ICALPARSER_ERROR;
-	icalmemory_free_buffer(str), str = NULL;
+	icalmemory_free_buffer(str);
+	str = NULL;
 	return 0;
     }
 
@@ -817,11 +857,13 @@ icalcomponent* icalparser_add_line(icalparser* parser,
 	    
 	tail = 0;
 	parser->state = ICALPARSER_ERROR;
-	icalmemory_free_buffer(str), str = NULL;
+	icalmemory_free_buffer(str);
+	str = NULL;
 	return 0;
     }
 
-    icalmemory_free_buffer(str), str = NULL;
+    icalmemory_free_buffer(str);
+    str = NULL;
 
     /**********************************************************************
      * Handle parameter values
@@ -839,8 +881,7 @@ icalcomponent* icalparser_add_line(icalparser* parser,
 	}
 
 	str = parser_get_next_parameter(end,&end);
-	strptr = str;
-	str = strstrip (str);
+	strstriplt(str);
 	if (str != 0){
 	    char* name;
 	    char* pvalue;
@@ -869,16 +910,25 @@ icalcomponent* icalparser_add_line(icalparser* parser,
 		    icalparameter_set_xname(param,name);
 		    icalparameter_set_xvalue(param,pvalue);
 		}
-		icalmemory_free_buffer(buf_value), buf_value = NULL;
+		icalmemory_free_buffer(buf_value);
+		buf_value = NULL;
 
 	    } else if (kind != ICAL_NO_PARAMETER){
 		param = icalparameter_new_from_value_string(kind,pvalue);
 
-		icalmemory_free_buffer(buf_value), buf_value = NULL;
+		icalmemory_free_buffer(buf_value);
+		buf_value = NULL;
 
 	    } else {
 		/* Error. Failed to parse the parameter*/
 		/* 'tail' defined above */
+
+                /* Change for mozilla */
+                /* have the option of being flexible towards unsupported parameters */
+                #ifndef ICAL_ERRORS_ARE_FATAL
+                continue;
+                #endif
+
 		insert_error(tail, str, "Cant parse parameter name",
 			     ICAL_XLICERRORTYPE_PARAMETERNAMEPARSEERROR);
 		tail = 0;
@@ -894,9 +944,12 @@ icalcomponent* icalparser_add_line(icalparser* parser,
 		tail = 0;
 		parser->state = ICALPARSER_ERROR;
 		
-		icalmemory_free_buffer(buf_value), buf_value = NULL;
-		icalmemory_free_buffer(name), name = NULL;
-		icalmemory_free_buffer(strptr), strptr = str = NULL;
+		icalmemory_free_buffer(buf_value);
+		buf_value = NULL;
+		icalmemory_free_buffer(name);
+		name = NULL;
+		icalmemory_free_buffer(str);
+		str = NULL;
   	  	
 		continue;
 	    }
@@ -929,23 +982,28 @@ icalcomponent* icalparser_add_line(icalparser* parser,
 		    tail = 0;
 		    parser->state = ICALPARSER_ERROR;
 
-			icalmemory_free_buffer(name), name = NULL;
-			icalmemory_free_buffer(strptr), strptr = str = NULL;
+			icalmemory_free_buffer(name);
+			name = NULL;
+			icalmemory_free_buffer(str);
+			str = NULL;
 		    continue;
 		} 
 	    }
-		icalmemory_free_buffer(name), name = NULL;
+		icalmemory_free_buffer(name);
+		name = NULL;
 
 	    /* Everything is OK, so add the parameter */
 	    icalproperty_add_parameter(prop,param);
 	    tail = 0;
-		icalmemory_free_buffer(strptr), strptr = str = NULL;
+	    icalmemory_free_buffer(str);
+	    str = NULL;
 
 	} else { /* if ( str != 0)  */
-	    /* If we did not get a param string, go on to looking
-	       for a value */
-			if (strptr != NULL)
-					icalmemory_free_buffer(strptr), strptr = str = NULL;
+	    /* If we did not get a param string, go on to looking for a value */
+		if (str != NULL) {
+			icalmemory_free_buffer(str);
+			str = NULL;
+		}
 	    break;
 	} /* if ( str != 0)  */
 
@@ -961,9 +1019,33 @@ icalcomponent* icalparser_add_line(icalparser* parser,
 
     vcount=0;
     while(1) {
-	str = parser_get_next_value(end,&end, value_kind);
-	strptr = str;
-	str = strstrip (str);
+        /* Only some properies can have multiple values. This list was taken
+           from rfc2445. Also added the x-properties, because the spec actually
+           says that commas should be escaped. For x-properties, other apps may
+           depend on that behaviour
+        */
+        switch (prop_kind) {
+            case ICAL_X_PROPERTY:
+            case ICAL_CATEGORIES_PROPERTY:
+            case ICAL_RESOURCES_PROPERTY:
+            /* Referring to RFC 2445, section 4.8.5.3 and section 4.8.5.1:
+               RDATE and EXDATE can specify a list of dates/date-times/periods.
+            */
+            case ICAL_RDATE_PROPERTY:
+            case ICAL_EXDATE_PROPERTY:
+            /* Referring to RFC 2445, section 4.8.2.6 Free/Busy Time:
+               The "FREEBUSY" property can specify more than one value, separated by
+               the COMMA character (US-ASCII decimal 44).
+            */
+            case ICAL_FREEBUSY_PROPERTY:
+                 str = parser_get_next_value(end,&end, value_kind);
+		 strstriplt (str);
+                 break;
+            default:
+                 str = icalparser_get_value(end, &end, value_kind);
+		 strstriplt (str);
+                 break;
+        }
 
 	if (str != 0){
 		
@@ -1000,18 +1082,22 @@ icalcomponent* icalparser_add_line(icalparser* parser,
 		tail = 0;
 		parser->state = ICALPARSER_ERROR;
 	
-		icalmemory_free_buffer(strptr), str = NULL, strptr = NULL;
+		icalmemory_free_buffer(str);
+		str = NULL;
 		return 0;
 		    
 	    } else {
 		vcount++;
 		icalproperty_set_value(prop, value);
 	    }
- 	    icalmemory_free_buffer(strptr), str = NULL, strptr = NULL;
+ 	    icalmemory_free_buffer(str);
+	    str = NULL;
 
 	} else {
-			if (strptr != NULL)
-					icalmemory_free_buffer(strptr), str = NULL, strptr = NULL;
+		if (str != NULL) {
+			icalmemory_free_buffer(str);
+			str = NULL;
+		}
 			
 	    if (vcount == 0){
 		char temp[200]; /* HACK */
