@@ -879,31 +879,58 @@ int icaltime_compare(const struct icaltimetype a_in, const struct icaltimetype b
 int
 icaltime_compare_date_only(const struct icaltimetype a_in, const struct icaltimetype b_in)
 {
-    int retval;
     struct icaltimetype a, b;
+    icaltimezone *tz = icaltimezone_get_utc_timezone();
 
-    a = icaltime_convert_to_zone(a_in, icaltimezone_get_utc_timezone());
-    b = icaltime_convert_to_zone(b_in, icaltimezone_get_utc_timezone());
+    a = icaltime_convert_to_zone(a_in, tz);
+    b = icaltime_convert_to_zone(b_in, tz);
 
     if (a.year > b.year)
-	retval = 1;
+	return 1;
     else if (a.year < b.year)
-	retval = -1;
+	return -1;
 
-    else if (a.month > b.month)
-	retval = 1;
+    if (a.month > b.month)
+	return 1;
     else if (a.month < b.month)
-	retval = -1;
+	return -1;
 
-    else if (a.day > b.day)
-	retval = 1;
+    if (a.day > b.day)
+	return 1;
     else if (a.day < b.day)
-	retval = -1;
+	return -1;
 
-    else
-	retval = 0;
+    return 0;
+}
 
-    return retval;
+/**
+ *	like icaltime_compare, but only use the date parts; accepts timezone.
+ */
+
+int
+icaltime_compare_date_only_tz(const struct icaltimetype a_in, const struct icaltimetype b_in, icaltimezone *tz)
+{
+    struct icaltimetype a, b;
+
+    a = icaltime_convert_to_zone(a_in, tz);
+    b = icaltime_convert_to_zone(b_in, tz);
+
+    if (a.year > b.year)
+	return 1;
+    else if (a.year < b.year)
+	return -1;
+
+    if (a.month > b.month)
+	return 1;
+    else if (a.month < b.month)
+	return -1;
+
+    if (a.day > b.day)
+	return 1;
+    else if (a.day < b.day)
+	return -1;
+
+    return 0;
 }
 
 /* These are defined in icalduration.c:
